package com.baidu.rtc.videoroom;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.media.AudioAttributes;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Build;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;

import com.baidu.rtc.BaiduRtcRoom;
import com.baidu.rtc.BaiduRtcRoomDelegate;
import com.baidu.rtc.RtcParameterSettings;
import com.baidu.rtc.videoroom.model.RoomData;
import com.baidu.rtc.videoroom.utils.Utils;

public class BRtcWrapper {
    private static final String TAG = BRtcWrapper.class.getSimpleName();
    private BaiduRtcRoom mAudioRoom;
    private RoomData mRoomData;
    private Context mContext = null;
    private Handler mHandler = null;
    private volatile boolean mDestroy = false;
    private String mInfo = "error";
    private RtcListener mListener = null;


    public BRtcWrapper(RtcListener listener) {
        mListener = listener;
    }

    public void onCreate(RoomData data, Context context) {
        mDestroy = false;
        mContext = context;
        mHandler = new Handler();
        mRoomData = data;
        hasNetwork = syncNetworkState();
        Log.i(TAG, "BaiduRTC(BRTC) SDK version is: " + BaiduRtcRoom.version());

        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(ConnectivityManager.CONNECTIVITY_ACTION);
        context.registerReceiver(networkChangeReceiver, intentFilter);

        if (!hasNetwork) {
            makeToast("no network");
            return;
        }
        BaiduRtcRoom.WriteLogConfig writeLogConfig = new BaiduRtcRoom.WriteLogConfig();
        writeLogConfig.logName = "bdrtc_alog";
        writeLogConfig.logPath = Environment.getExternalStorageDirectory().getAbsolutePath() + "/";
        writeLogConfig.maxNum = 5;
        writeLogConfig.maxBytes = 1000 * 1024;
        writeLogConfig.enableWriteFile = false; // 调试中可开启写文件能力..
        BaiduRtcRoom.setVerbose(true);
        BaiduRtcRoom.setWriteLogConfig(writeLogConfig);
        loginTask.run();


    }

    public void onDestroy() {
        if (mDestroy) {
            Log.i(TAG, " RTC already destroy");
            return;
        }
        mIsConnected = false;
        mDestroy = true;

        mHandler.removeCallbacksAndMessages(null);
        mHandler.post(new Runnable() {
            @Override
            public void run() {
                if (mAudioRoom != null) {
                    mAudioRoom.destroy();
                    mAudioRoom = null;
                }
            }
        });

        if (mContext != null) {
            mContext.unregisterReceiver(networkChangeReceiver);
        }
        Log.i(TAG, " RTC destroy");
    }

    public void muteMicphone(boolean muted) {
        if (mAudioRoom != null) {
            mAudioRoom.muteMicphone(muted);
        }
    }

    public void switchLoundSpeaker() {
        if (mAudioRoom != null) {
            mAudioRoom.switchLoundSpeaker();
        }
    }

    public void logout() {
        if (mAudioRoom != null) {
            mAudioRoom.logoutRtcRoom();
        }
    }

    public void destroy() {
        if (mAudioRoom != null) {
            mAudioRoom.destroy();
        }
    }

    public boolean isConnected() {
        return mIsConnected;
    }

    public BaiduRtcRoom.RtcRoomAudioLevel getLocalAudioLevel() {
        return mAudioRoom.getLocalAudioLevel();
    }

    public BaiduRtcRoom.RtcRoomAudioLevel[] getRemoteAudioLevels() {
        return mAudioRoom.getRemoteAudioLevels();
    }

    private Runnable loginTask = new Runnable() {
        @Override
        public void run() {
            if (mDestroy) {
                Log.i(TAG, "try login rtc room, but recv destroy signal!");
                return;
            }
            Log.e(TAG, "try login rtc room");
            if (mAudioRoom != null) {
                Log.e(TAG, "try login, logout room");
                mAudioRoom.logoutRtcRoom();
                mAudioRoom.destroy();
                mIsConnected = false;
            }
            Log.e(TAG, "try login, init RtcRoom");
            mAudioRoom = BaiduRtcRoom.initWithAppID(mContext, mRoomData.mAppId, mRoomData.mTokenStr);

            if (mRoomData.mReportStreamMonitorData) {
                mAudioRoom.enableStatsToServer(true, true, "online");
            }
            mAudioRoom.enableAns(mRoomData.mDisableBuildInNs);
            mAudioRoom.setBaiduRtcRoomDelegate(delegate);
            mAudioRoom.setMediaServerURL(mRoomData.mMediaServer);


            if (mRoomData.mRtmpUrl.contains("rtmp")) {
                mAudioRoom.setLiveStreamingURL(mRoomData.mRtmpUrl);
                mAudioRoom.setLiveStreamingMix(mRoomData.mRtmpMix);
            }

            if (mRoomData.mRecording) {
                mAudioRoom.setRecording(mRoomData.mRecording);
            }

            mHandler.postDelayed(new Runnable() {
                @Override
                public void run() {
                    doLogin();
                }
            }, 100);

        }
    };

    private void exist() {
        if (mHandler == null) {
            return;
        }
        mHandler.post(new Runnable() {
            @Override
            public void run() {
                mListener.requestExist();
            }
        });
    }

    private void makeToast(String txt) {
        if (mHandler == null) {
            return;
        }
        mHandler.post(new Runnable() {
            @Override
            public void run() {
                Utils.makeToast(mContext, txt);
            }
        });
    }


    boolean mIsConnected = false;
    private BaiduRtcRoomDelegate delegate = new BaiduRtcRoomDelegate() {

        @Override
        public void onConnectionLost(int code) {
            Log.e(TAG, "onConnectionLost code: " + code);
            mIsConnected = false;
            if (mDestroy) {
                return;
            }
            if (hasNetwork) {
                mHandler.post(loginTask);
            }
            mListener.onConnectionLost(code);
        }

        @Override
        public void onConnectionStateChanged(int newState, int reason) {
            Log.e(TAG, "onConnectionStateChanged newState: " + newState + ", reason:" + reason);
        }



        @Override
        public void onLoginRtcRoom(int code, long roomId) {
            Log.e(TAG, "onLoginRtcRoom code: " + code + ", roomId:" + roomId);
            mListener.onLoginRtcRoom(code, roomId);
            if (code == 0) {
                mIsConnected = true;
            } else {
                mIsConnected = false;
                mInfo = mContext.getString(R.string.login_room_failed);

                makeToast(mInfo);
                exist();
            }
        }

        @Override
        public void onUserJoined(long uid, String displayName) {
            Log.e(TAG, "onRoomEventUpdate onUserJoinedRoom id: " + uid + ",name: " + displayName);
        }

        @Override
        public void onUserOffline(long uid, String msg) {
            Log.e(TAG, "onRoomEventUpdate onUserOffline id: " + uid);
        }

        @Override
        public void onUserMessage(long uid, String msg) {
            Log.e(TAG, "onRoomEventUpdate onUserMessage id: " + uid + " msg: " + msg);

            if (msg.contains("ping getattribute")) {
                mAudioRoom.getUserAttribute(uid);
            }
            if (msg.contains("ping sendmessage")) {
                mAudioRoom.sendMessageToUser("hello", 0);
            }
            if (msg.contains("ping setattribute")) {
                mAudioRoom.setUserAttribute("{'name':'jim','tel':'123456789'}");
            }
            if (msg.contains("ping disabandroom")) {
                mAudioRoom.disbandRoom();
            }
            if (msg.contains("ping shutupuser")) {
                mAudioRoom.shutUpUserWithId(123456, true);
            }
            if (msg.contains("ping unshutupuser")) {
                mAudioRoom.shutUpUserWithId(123456, false);
            }
            if (msg.contains("ping kickoffuser")) {
                mAudioRoom.kickOffUserWithId(123456);
            }
        }

        @Override
        public void onUserAttribute(long uid, String msg) {
            Log.e(TAG, "onUserAttribute  id: " + uid + " attribute: " + msg);
        }

        @Override
        public void onFirstRemoteAudio(long uid, String msg) {
            Log.e(TAG, "onFirstRemoteAudio  uid: " + uid + " msg: " + msg);
        }

        @Override
        public void onRemoteAudioOffline(long uid, String msg) {
            Log.e(TAG, "onRemoteAudioOffline  uid: " + uid + " msg: " + msg);
        }

        @Override
        public void onRemoteStreamComing(long uid, String name) {
            Log.e(TAG, "onRemoteStreamComing : Coming UID " + uid + " Display:" + name);
            mListener.onRemoteStreamComing(uid, name);
        }

        @Override
        public void onRemoteStreamLeaving(long uid) {
            Log.e(TAG, "onRemoteStreamLeaving : Leaving UID " + uid);
            mListener.onRemoteStreamLeaving(uid);
        }

        @Override
        public void onSubscriberMediaStateUpdate(int connecStates) {
            Log.i(TAG, "onSubscriberMediaStateUpdate is: " + connecStates);
        }

        @Override
        public void onPublisherMediaStateUpdate(int state) {
            Log.i(TAG, "onPublisherMediaStateUpdate is: " + state);
            if (state == RTC_STATE_SENDING_MEDIA_OK) {
                makeToast("send media ok");
            } else if (state == RTC_STATE_SENDING_MEDIA_FAILED ||
                    state == RTC_STATE_PUBLISHER_STREAM_DOWN) {
                makeToast("send media fail");
            }
            mListener.onPublisherMediaStateUpdate(state);
        }


        @Override
        public void onError(int errorInfo) {
            Log.e(TAG, "onErrorInfoUpdate, info:" + errorInfo);
            /**
             * 用户不存在 订阅和取消订阅导致错误
             */
            if (errorInfo == ErrorCode.ERROR_UNSUBSCRIBE_USER_NOT_SUBSCRIBED ||
                    errorInfo == ErrorCode.ERROR_SUBSCRIBER_USER_NOT_EXISTS) {
                return;
            }
            makeToast("onError, errorCode" + errorInfo);
            exist();
        }

        @Override
        public void onWarn(int errorInfo) {
            Log.e(TAG, "onWarn, info:" + errorInfo);
        }
    };

    private boolean hasNetwork = true;

    private boolean syncNetworkState() {
        ConnectivityManager connManager = (ConnectivityManager)
                mContext.getSystemService(Context.CONNECTIVITY_SERVICE);
        if (null == connManager) {
            return false;
        }
        NetworkInfo activeNetInfo = connManager.getActiveNetworkInfo();
        if (activeNetInfo == null || !activeNetInfo.isAvailable()) {
            return false;
        }
        return true;
    }

    private BroadcastReceiver networkChangeReceiver = new BroadcastReceiver() {
        int lastNetworkType = -1;

        @Override
        public void onReceive(Context context, Intent intent) {
            Log.e(TAG, "Network connectivity change, " + intent);
            ConnectivityManager connManager = (ConnectivityManager)
                    context.getSystemService(Context.CONNECTIVITY_SERVICE);
            if (null == connManager) {
                lastNetworkType = 0;
                hasNetwork = false;
                return;
            }
            NetworkInfo activeNetInfo = connManager.getActiveNetworkInfo();
            if (activeNetInfo == null || !activeNetInfo.isAvailable()) {
                lastNetworkType = 0;
                hasNetwork = false;
                return;
            }
            if (lastNetworkType >= 0 && !mDestroy) {
                mHandler.postDelayed(loginTask, 200);
            }
            lastNetworkType = activeNetInfo.getType();
            hasNetwork = true;
        }
    };

    private void doLogin() {
        RtcParameterSettings cfg = RtcParameterSettings.getDefaultSettings();

        cfg.MediaServerIP = mRoomData.mMediaServerIP;
        cfg.HasData = false;

        if (mRoomData.mAudioOnly) {
            cfg.HasVideo = false;
        }

        cfg.ConnectionTimeoutMs = 5000;
        cfg.ReadTimeoutMs = 5000;

        if (Build.MANUFACTURER.contains("Ainemo")
                || Build.MODEL.contains("NV6001")
                || Build.MODEL.contains("NV6101")
                || Build.MODEL.contains("NV2001")
                || Build.MODEL.contains("NV5001")) { // Duer 1s 1C 1A 1L, legacy
            cfg.AudioFrequency = 16000;
            cfg.AudioChannel = 2;
            cfg.AudioContentType = AudioAttributes.CONTENT_TYPE_MUSIC;
        }

        cfg.AutoPublish = true;       // default is true.  for mVideoRoom.startPublish() set to false.
        cfg.AutoSubScribe = true;     // default is true.  for mVideoRoom.subscribeStreaming() set to false.

        mAudioRoom.setParamSettings(cfg,
                RtcParameterSettings.RtcParamSettingType.RTC_PARAM_SETTINGS_ALL);
        mAudioRoom
                .loginRtcRoomWithRoomName(mRoomData.mRoomName,
                        java.lang.Long.parseLong(mRoomData.mUserId),
                        mRoomData.mUserName, true);

        mListener.startLogin();
    }


    public interface  RtcListener {
        void startLogin();

        void onLoginRtcRoom(int code, long roomId);

        void onConnectionLost(int code);

        void onPublisherMediaStateUpdate(int state);

        void onRemoteStreamComing(long uid, String name);

        void onRemoteStreamLeaving(long uid);

        void requestExist();
    }
}
