package com.baidu.rtc.videoroom.model;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.widget.SeekBar;
import android.widget.TextView;

import com.baidu.rtc.BaiduRtcRoom;
import com.baidu.rtc.videoroom.R;
import com.zhy.adapter.recyclerview.CommonAdapter;
import com.zhy.adapter.recyclerview.base.ViewHolder;

import java.util.ArrayList;

public class UserModel {
    private static final String TAG = UserModel.class.getSimpleName();
    private static final int USER_LOGIN_TIMEOUT = 5000;
    private ArrayList<UserInfo> mUserInfoList = new ArrayList<>();
    private CommonAdapter<UserInfo> mAdapter;
    private Handler mHandler = null;

    /** 用户角色 */
    public enum UserRole {
        LOCAL,
        REMOTE,
    }

    public enum RoomState {
        IDLE,
        LOGIN,
        OK,
        FAIL
    }

    public enum StreamState {
        IDLE,
        ICE_CONNECTED,
        ICE_DIS_CONNECTED,
        WEBRTC_UP,
        SEND_MEDIA_OK,
        SEND_MEDIA_FAIL,
    }


    /** 用户信息数据 */
    public static class UserInfo {
        public long userId = 0;
        public int audioVolume = 0;
        public RoomState roomState = RoomState.IDLE;
        public StreamState streamState = StreamState.IDLE;
        public UserRole role;
        long lostTs = 0;
    }

    public void add(UserInfo info) {
        if (info == null) {
            return;
        }
        boolean bfind = false;
        for (UserInfo u : mUserInfoList) {
            if (u.userId == info.userId) {
                bfind = true;
                break;
            }
        }
        if (!bfind) {
            Log.i(TAG, "add user, uid:" + info.userId);
            mUserInfoList.add(info);
        }
    }

    public void add(long uid) {
        Log.i(TAG, "add remote user, uid:" + uid);
        UserModel.UserInfo userInfo = new UserModel.UserInfo();
        userInfo.userId = Long.valueOf(uid);
        userInfo.role = UserRole.REMOTE;
        add(userInfo);
    }

    public void remove(long uid) {
        UserInfo target = null;
        for (UserInfo info : mUserInfoList) {
            if (info.userId == uid) {
                target = info;
                break;
            }
        }
        if (target != null) {
            mUserInfoList.remove(target);
        }
    }

    public UserInfo get(long uid) {
        UserInfo target = null;
        for (UserInfo info : mUserInfoList) {
            if (info.userId == uid) {
                target = info;
                break;
            }
        }
        return target;
    }

    /** 更新远端音频level */
    public void updateVolume(BaiduRtcRoom.RtcRoomAudioLevel[] levels) {
        ArrayList<UserInfo> delList = new ArrayList<>();
        long cts = System.currentTimeMillis();
        for (UserInfo info : mUserInfoList) {
            boolean bfind = false;
            for (BaiduRtcRoom.RtcRoomAudioLevel level : levels) {
                if (level.userId == info.userId) {
                    info.audioVolume = level.volumeLevel;
                    info.lostTs = 0;
                    bfind = true;
                    break;
                }
            }

            /** 远端超过5秒未链接，直接移出用户数据 */
            if (!bfind && info.role == UserRole.REMOTE) {

                if (info.lostTs == 0) {
                    info.lostTs = cts;
                } else if (info.lostTs > 0 && info.lostTs + USER_LOGIN_TIMEOUT < cts){
                    delList.add(info);
                }
            }
        }

        for (UserInfo info : delList) {
            Log.i(TAG, "remove remote user, uid:" + info.userId + ", lostTs:" + info.lostTs);
            mUserInfoList.remove(info);
        }
    }

    public void update() {
        if (mHandler == null) {
            mHandler = new android.os.Handler();
        }
        if (mAdapter != null) {
            if (Looper.myLooper() == Looper.getMainLooper()) {
                mAdapter.notifyDataSetChanged();
            } else {
                mHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        mAdapter.notifyDataSetChanged();
                    }
                });
            }
        }
    }

    public void initView(Context context, RecyclerView view){
        mAdapter = new CommonAdapter<UserInfo>(context, R.layout.remote_user, mUserInfoList) {
            @Override
            protected void convert(ViewHolder viewHolder, UserInfo userInfo, int i) {
                ((TextView) viewHolder.getView(R.id.remoteUserIdView)).setText(
                        context.getString(R.string.list_user_item_name,
                                userInfo.userId + ""));
                ((TextView) viewHolder.getView(R.id.user_room_state)).setText(
                        context.getString(R.string.list_user_item_room_state,
                                userInfo.roomState.name().toLowerCase()));
                ((TextView) viewHolder.getView(R.id.user_stream_state)).setText(
                        context.getString(R.string.list_user_item_stream_state,
                                userInfo.streamState.name().toLowerCase()));
                ( (SeekBar) viewHolder.getView(R.id.sb_normal)).setMax(100);
                ( ( SeekBar ) viewHolder.getView(R.id.sb_normal)).setProgress(userInfo.audioVolume);
            }
        };
        view.setAdapter(mAdapter);
        Log.i(TAG, "initView :" + view);
    }
}
